@@ -32,7 +32,7 @@ namespace {
 TEST_F(AArch64GISelMITest, MatchIntConstant) {
   setUp();
   if (!TM)
-    return;
+    GTEST_SKIP();
   auto MIBCst = B.buildConstant(LLT::scalar(64), 42);
   int64_t Cst;
   bool match = mi_match(MIBCst.getReg(0), *MRI, m_ICst(Cst));
@@ -43,7 +43,7 @@ TEST_F(AArch64GISelMITest, MatchIntConstant) {
 TEST_F(AArch64GISelMITest, MatchIntConstantRegister) {
   setUp();
   if (!TM)
-    return;
+    GTEST_SKIP();
   auto MIBCst = B.buildConstant(LLT::scalar(64), 42);
   std::optional<ValueAndVReg> Src0;
   bool match = mi_match(MIBCst.getReg(0), *MRI, m_GCst(Src0));
@@ -54,14 +54,14 @@ TEST_F(AArch64GISelMITest, MatchIntConstantRegister) {
 TEST_F(AArch64GISelMITest, MatchIntConstantSplat) {
   setUp();
   if (!TM)
-    return;
+    GTEST_SKIP();
 
   LLT s64 = LLT::scalar(64);
   LLT v2s64 = LLT::fixed_vector(2, s64);
   LLT v4s64 = LLT::fixed_vector(4, s64);
 
   MachineInstrBuilder FortyTwoSplat =
-      B.buildSplatVector(v4s64, B.buildConstant(s64, 42));
+      B.buildSplatBuildVector(v4s64, B.buildConstant(s64, 42));
   int64_t Cst;
   EXPECT_TRUE(mi_match(FortyTwoSplat.getReg(0), *MRI, m_ICstOrSplat(Cst)));
   EXPECT_EQ(Cst, 42);
@@ -79,7 +79,7 @@ TEST_F(AArch64GISelMITest, MatchIntConstantSplat) {
 TEST_F(AArch64GISelMITest, MachineInstrPtrBind) {
   setUp();
   if (!TM)
-    return;
+    GTEST_SKIP();
   auto MIBAdd = B.buildAdd(LLT::scalar(64), Copies[0], Copies[1]);
   // Test 'MachineInstr *' bind.
   // Default mi_match.
@@ -110,7 +110,7 @@ TEST_F(AArch64GISelMITest, MachineInstrPtrBind) {
 TEST_F(AArch64GISelMITest, MatchBinaryOp) {
   setUp();
   if (!TM)
-    return;
+    GTEST_SKIP();
   LLT s32 = LLT::scalar(32);
   LLT s64 = LLT::scalar(64);
   LLT p0 = LLT::pointer(0, 64);
@@ -224,6 +224,32 @@ TEST_F(AArch64GISelMITest, MatchBinaryOp) {
   auto MIBAddCst = B.buildAdd(s64, MIBCst, Copies[0]);
   auto MIBUnmerge = B.buildUnmerge({s32, s32}, B.buildConstant(s64, 42));
 
+  // Match min/max, and make sure they're commutative.
+  auto SMin = B.buildSMin(s64, Copies[2], MIBAdd);
+  EXPECT_TRUE(mi_match(SMin.getReg(0), *MRI,
+                       m_GSMin(m_GAdd(m_Reg(Src1), m_Reg(Src2)), m_Reg(Src0))));
+  EXPECT_EQ(Src0, Copies[2]);
+  EXPECT_EQ(Src1, Copies[0]);
+  EXPECT_EQ(Src2, Copies[1]);
+  auto SMax = B.buildSMax(s64, Copies[2], MIBAdd);
+  EXPECT_TRUE(mi_match(SMax.getReg(0), *MRI,
+                       m_GSMax(m_GAdd(m_Reg(Src1), m_Reg(Src2)), m_Reg(Src0))));
+  EXPECT_EQ(Src0, Copies[2]);
+  EXPECT_EQ(Src1, Copies[0]);
+  EXPECT_EQ(Src2, Copies[1]);
+  auto UMin = B.buildUMin(s64, Copies[2], MIBAdd);
+  EXPECT_TRUE(mi_match(UMin.getReg(0), *MRI,
+                       m_GUMin(m_GAdd(m_Reg(Src1), m_Reg(Src2)), m_Reg(Src0))));
+  EXPECT_EQ(Src0, Copies[2]);
+  EXPECT_EQ(Src1, Copies[0]);
+  EXPECT_EQ(Src2, Copies[1]);
+  auto UMax = B.buildUMax(s64, Copies[2], MIBAdd);
+  EXPECT_TRUE(mi_match(UMax.getReg(0), *MRI,
+                       m_GUMax(m_GAdd(m_Reg(Src1), m_Reg(Src2)), m_Reg(Src0))));
+  EXPECT_EQ(Src0, Copies[2]);
+  EXPECT_EQ(Src1, Copies[0]);
+  EXPECT_EQ(Src2, Copies[1]);
+
   // m_BinOp with opcode.
   // Match binary instruction, opcode and its non-commutative operands.
   match = mi_match(MIBAddCst, *MRI,
@@ -282,7 +308,7 @@ TEST_F(AArch64GISelMITest, MatchBinaryOp) {
 TEST_F(AArch64GISelMITest, MatchICmp) {
   setUp();
   if (!TM)
-    return;
+    GTEST_SKIP();
 
   const LLT s1 = LLT::scalar(1);
   auto CmpEq = B.buildICmp(CmpInst::ICMP_EQ, s1, Copies[0], Copies[1]);
@@ -307,7 +333,7 @@ TEST_F(AArch64GISelMITest, MatchICmp) {
 TEST_F(AArch64GISelMITest, MatchFCmp) {
   setUp();
   if (!TM)
-    return;
+    GTEST_SKIP();
 
   const LLT s1 = LLT::scalar(1);
   auto CmpEq = B.buildFCmp(CmpInst::FCMP_OEQ, s1, Copies[0], Copies[1]);
@@ -332,7 +358,7 @@ TEST_F(AArch64GISelMITest, MatchFCmp) {
 TEST_F(AArch64GISelMITest, MatcCommutativeICmp) {
   setUp();
   if (!TM)
-    return;
+    GTEST_SKIP();
   const LLT s1 = LLT::scalar(1);
   Register LHS = Copies[0];
   Register RHS = Copies[1];
@@ -361,7 +387,7 @@ TEST_F(AArch64GISelMITest, MatcCommutativeICmp) {
 TEST_F(AArch64GISelMITest, MatcCommutativeFCmp) {
   setUp();
   if (!TM)
-    return;
+    GTEST_SKIP();
   const LLT s1 = LLT::scalar(1);
   Register LHS = Copies[0];
   Register RHS = Copies[1];
@@ -390,7 +416,7 @@ TEST_F(AArch64GISelMITest, MatcCommutativeFCmp) {
 TEST_F(AArch64GISelMITest, MatchFPUnaryOp) {
   setUp();
   if (!TM)
-    return;
+    GTEST_SKIP();
 
   // Truncate s64 to s32.
   LLT s32 = LLT::scalar(32);
@@ -452,7 +478,7 @@ TEST_F(AArch64GISelMITest, MatchFPUnaryOp) {
 TEST_F(AArch64GISelMITest, MatchExtendsTrunc) {
   setUp();
   if (!TM)
-    return;
+    GTEST_SKIP();
 
   LLT s64 = LLT::scalar(64);
   LLT s32 = LLT::scalar(32);
@@ -499,7 +525,7 @@ TEST_F(AArch64GISelMITest, MatchExtendsTrunc) {
 TEST_F(AArch64GISelMITest, MatchSpecificType) {
   setUp();
   if (!TM)
-    return;
+    GTEST_SKIP();
 
   // Try to match a 64bit add.
   LLT s64 = LLT::scalar(64);
@@ -536,7 +562,7 @@ TEST_F(AArch64GISelMITest, MatchSpecificType) {
 TEST_F(AArch64GISelMITest, MatchCombinators) {
   setUp();
   if (!TM)
-    return;
+    GTEST_SKIP();
 
   LLT s64 = LLT::scalar(64);
   LLT s32 = LLT::scalar(32);
@@ -570,12 +596,17 @@ TEST_F(AArch64GISelMITest, MatchCombinators) {
 TEST_F(AArch64GISelMITest, MatchMiscellaneous) {
   setUp();
   if (!TM)
-    return;
+    GTEST_SKIP();
 
   LLT s64 = LLT::scalar(64);
   auto MIBAdd = B.buildAdd(s64, Copies[0], Copies[1]);
   Register Reg = MIBAdd.getReg(0);
 
+  // Extract the type.
+  LLT Ty;
+  EXPECT_TRUE(mi_match(Reg, *MRI, m_GAdd(m_Type(Ty), m_Reg())));
+  EXPECT_EQ(Ty, s64);
+
   // Only one use of Reg.
   B.buildCast(LLT::pointer(0, 32), MIBAdd);
   EXPECT_TRUE(mi_match(Reg, *MRI, m_OneUse(m_GAdd(m_Reg(), m_Reg()))));
@@ -597,7 +628,7 @@ TEST_F(AArch64GISelMITest, MatchMiscellaneous) {
 TEST_F(AArch64GISelMITest, MatchSpecificConstant) {
   setUp();
   if (!TM)
-    return;
+    GTEST_SKIP();
 
   // Basic case: Can we match a G_CONSTANT with a specific value?
   auto FortyTwo = B.buildConstant(LLT::scalar(64), 42);
@@ -619,13 +650,13 @@ TEST_F(AArch64GISelMITest, MatchSpecificConstant) {
 TEST_F(AArch64GISelMITest, MatchSpecificConstantSplat) {
   setUp();
   if (!TM)
-    return;
+    GTEST_SKIP();
 
   LLT s64 = LLT::scalar(64);
   LLT v4s64 = LLT::fixed_vector(4, s64);
 
   MachineInstrBuilder FortyTwoSplat =
-      B.buildSplatVector(v4s64, B.buildConstant(s64, 42));
+      B.buildSplatBuildVector(v4s64, B.buildConstant(s64, 42));
   MachineInstrBuilder FortyTwo = B.buildConstant(s64, 42);
 
   EXPECT_TRUE(mi_match(FortyTwoSplat.getReg(0), *MRI, m_SpecificICstSplat(42)));
@@ -649,13 +680,13 @@ TEST_F(AArch64GISelMITest, MatchSpecificConstantSplat) {
 TEST_F(AArch64GISelMITest, MatchSpecificConstantOrSplat) {
   setUp();
   if (!TM)
-    return;
+    GTEST_SKIP();
 
   LLT s64 = LLT::scalar(64);
   LLT v4s64 = LLT::fixed_vector(4, s64);
 
   MachineInstrBuilder FortyTwoSplat =
-      B.buildSplatVector(v4s64, B.buildConstant(s64, 42));
+      B.buildSplatBuildVector(v4s64, B.buildConstant(s64, 42));
   MachineInstrBuilder FortyTwo = B.buildConstant(s64, 42);
 
   EXPECT_TRUE(
@@ -680,7 +711,7 @@ TEST_F(AArch64GISelMITest, MatchSpecificConstantOrSplat) {
 TEST_F(AArch64GISelMITest, MatchZeroInt) {
   setUp();
   if (!TM)
-    return;
+    GTEST_SKIP();
   auto Zero = B.buildConstant(LLT::scalar(64), 0);
   EXPECT_TRUE(mi_match(Zero.getReg(0), *MRI, m_ZeroInt()));
 
@@ -691,7 +722,7 @@ TEST_F(AArch64GISelMITest, MatchZeroInt) {
 TEST_F(AArch64GISelMITest, MatchAllOnesInt) {
   setUp();
   if (!TM)
-    return;
+    GTEST_SKIP();
   auto AllOnes = B.buildConstant(LLT::scalar(64), -1);
   EXPECT_TRUE(mi_match(AllOnes.getReg(0), *MRI, m_AllOnesInt()));
 
@@ -702,7 +733,7 @@ TEST_F(AArch64GISelMITest, MatchAllOnesInt) {
 TEST_F(AArch64GISelMITest, MatchFPOrIntConst) {
   setUp();
   if (!TM)
-    return;
+    GTEST_SKIP();
 
   Register IntOne = B.buildConstant(LLT::scalar(64), 1).getReg(0);
   Register FPOne = B.buildFConstant(LLT::scalar(64), 1.0).getReg(0);
@@ -721,7 +752,7 @@ TEST_F(AArch64GISelMITest, MatchFPOrIntConst) {
 TEST_F(AArch64GISelMITest, MatchConstantSplat) {
   setUp();
   if (!TM)
-    return;
+    GTEST_SKIP();
 
   LLT s64 = LLT::scalar(64);
   LLT v2s64 = LLT::fixed_vector(2, 64);
@@ -811,7 +842,7 @@ TEST_F(AArch64GISelMITest, MatchConstantSplat) {
 TEST_F(AArch64GISelMITest, MatchNeg) {
   setUp();
   if (!TM)
-    return;
+    GTEST_SKIP();
 
   LLT s64 = LLT::scalar(64);
   auto Zero = B.buildConstant(LLT::scalar(64), 0);
@@ -843,7 +874,7 @@ TEST_F(AArch64GISelMITest, MatchNeg) {
 TEST_F(AArch64GISelMITest, MatchNot) {
   setUp();
   if (!TM)
-    return;
+    GTEST_SKIP();
 
   LLT s64 = LLT::scalar(64);
   auto AllOnes = B.buildConstant(LLT::scalar(64), -1);
@@ -876,7 +907,7 @@ TEST_F(AArch64GISelMITest, MatchNot) {
 TEST_F(AArch64GISelMITest, MatchSpecificReg) {
   setUp();
   if (!TM)
-    return;
+    GTEST_SKIP();
   auto Cst1 = B.buildConstant(LLT::scalar(64), 42);
   auto Cst2 = B.buildConstant(LLT::scalar(64), 314);
   Register Reg = Cst1.getReg(0);
@@ -889,6 +920,36 @@ TEST_F(AArch64GISelMITest, MatchSpecificReg) {
   EXPECT_TRUE(mi_match(Add.getReg(0), *MRI, m_GAdd(m_SpecificReg(Reg), m_Reg())));
 }
 
+TEST_F(AArch64GISelMITest, DeferredMatching) {
+  setUp();
+  if (!TM)
+    GTEST_SKIP();
+  auto s64 = LLT::scalar(64);
+  auto s32 = LLT::scalar(32);
+
+  auto Cst1 = B.buildConstant(s64, 42);
+  auto Cst2 = B.buildConstant(s64, 314);
+  auto Add = B.buildAdd(s64, Cst1, Cst2);
+  auto Sub = B.buildSub(s64, Add, Cst1);
+
+  auto TruncAdd = B.buildTrunc(s32, Add);
+  auto TruncSub = B.buildTrunc(s32, Sub);
+  auto NarrowAdd = B.buildAdd(s32, TruncAdd, TruncSub);
+
+  Register X;
+  EXPECT_TRUE(mi_match(Sub.getReg(0), *MRI,
+                       m_GSub(m_GAdd(m_Reg(X), m_Reg()), m_DeferredReg(X))));
+  LLT Ty;
+  EXPECT_TRUE(
+      mi_match(NarrowAdd.getReg(0), *MRI,
+               m_GAdd(m_GTrunc(m_Type(Ty)), m_GTrunc(m_DeferredType(Ty)))));
+
+  // Test commutative.
+  auto Add2 = B.buildAdd(s64, Sub, Cst1);
+  EXPECT_TRUE(mi_match(Add2.getReg(0), *MRI,
+                       m_GAdd(m_Reg(X), m_GSub(m_Reg(), m_DeferredReg(X)))));
+}
+
 } // namespace
 
 int main(int argc, char **argv) {
